#   Copyright Graham Jones 2014

#   This code is released under the Creative Commons 
#   Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)
#   licence, found here:
#   http://creativecommons.org/licenses/by-nc/4.0/


make.beast2xml.from.sim.data <- function(sim.options)
 {
  make.beast2xml.start("Simulated data made by seqgen and R code.",
                      paste0("For testing STACEY: ", sim.options$msc.model))
  
  make.beast2xml.data(sim.options)
  
  bigcomment("maps")
  make.beast2xml.maps()

  bigcomment("run")
  make.beast2xml.run(sim.options)
  
  make.beast2xml.end() 
}



make.beast2xml.start <- function(comment1, comment2) {
  catln(0, "<?xml",  namevalue("version", "1.0"),
        namevalue("encoding", "UTF-8"), 
        namevalue("standalone", "no"), "?>")
  nspce <- paste0("beast.core:",
                  "beast.evolution.alignment:",
                  "beast.evolution.tree:",
                  "beast.evolution.tree.coalescent:",
                  "beast.evolution.speciation:",
                  "beast.core.util:",
                  "beast.evolution.nuc:",
                  "beast.evolution.operators:",
                  "beast.evolution.sitemodel:",
                  "beast.evolution.substitutionmodel:",
                  "beast.evolution.likelihood")

  catln(0, "<beast", 
        namevalue("beautitemplate", ""),
        namevalue("beautistatus", ""),
        namevalue("namespace", nspce), 
        namevalue("version", "2.0"), ">")
}



make.beast2xml.maps <- function() {
  make.beast2xml.distribution.map("Beta")
  make.beast2xml.distribution.map("Exponential")
  make.beast2xml.distribution.map("InverseGamma")
  make.beast2xml.distribution.map("LogNormal", "LogNormalDistributionModel")
  make.beast2xml.distribution.map("Gamma")
  make.beast2xml.distribution.map("Uniform")
  make.beast2xml.distribution.map("LaplaceDistribution")
  make.beast2xml.distribution.map("OneOnX")
  make.beast2xml.distribution.map("Normal")
  catln(1, "<map name=\"prior\">beast.math.distributions.Prior</map>")
}

make.beast2xml.distribution.map <- function(distname, distspec=distname) {
  dm <- paste0("<map name=\"", distname, "\">beast.math.distributions.", distspec, "</map>")
  catln(1, dm)
}





make.beast2xml.run <- function(sim.options) {
  totalnofseqs <- get.totalnofseqs(sim.options$doing.olave, sim.options$nofgenes, sim.options$seqs.fpathbase)
  smcs <- all.smcs.from.taxa(sim.options$doing.olave, sim.options$msc.model, sim.options$taxa)
 
  catln(1, "<run", 
        namevalue("chainLength", sim.options$beastchainlength),
        id("mcmc"),
        spec("MCMC"),
        namevalue("storeEvery", "500000"),
        ">")
  
  bigcomment("state")
  make.beast2xml.state(sim.options, smcs)
  
  bigcomment("init")
  make.beast2xml.init(sim.options)
  #bigcomment("calculation node, unites smcTree with the gene trees")
  #make.beast2xml.smctreegtrees(sim.options) gone into make.coalescent.distribution()
  
  hugecomment("distribution")
  make.beast2xml.distribution(sim.options)
  
  bigcomment("operators")
  make.beast2xml.operators(sim.options, length(smcs), totalnofseqs)
  
  bigcomment("loggers")
  make.beast2xml.loggers(sim.options)
  
  catln(1, "</run>")
}



make.beast2xml.end <- function() {
  catln(0, "</beast>")
}




all.smcs.from.taxa <- function(doing.olave, msc.model, taxa) {
  if (doing.olave) {
    smcs <- rep("",40)
    for (sp in 1:8) {
      for (ind in 1:5) {
        smcs[5*(sp-1) + ind] <-
          paste0(c("a","b","c","d","e","f","g","h")[sp],
                 c("01","02","03","04","05")[ind])
      }
    }
    smcs
  } else {
    if (identical(msc.model, "STACEY")) {
      smcs <- rep("", length(taxa))
      for (t in 1:length(taxa)) {
        smcs[t] <- substr(taxa[t], 1, 1)
      }
      unique(smcs)
    } else if (identical(msc.model, "STACEYD")) {
      taxa
    }    
  }
}


taxa.subset <- function(letter, taxa) {
  txsubset <- character(0)
  for (j in 1:length(taxa)) {
    if (substr(taxa[j], 1, 1) == letter) {
      txsubset <- c(txsubset, taxa[j])
    }
  }
  txsubset
}




get.totalnofseqs <- function(doing.olave, nofgenes, seqs.fpathbase) {
  if (doing.olave) {
    totalnofseqs = 80 * nofgenes
  } else {
    totalnofseqs <- 0
    for (g in 1:nofgenes) {
      numastext <- as.character(g)
      textlines <- readLines(paste(seqs.fpathbase, numastext, ".txt", sep=""))
      n <- length(textlines)
      totalnofseqs <- totalnofseqs + n
    }
       
  }
  totalnofseqs 
}







