#   Copyright Graham Jones 2014

#   This code is released under the Creative Commons 
#   Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)
#   licence, found here:
#   http://creativecommons.org/licenses/by-nc/4.0/


make.beast2xml.data <- function(sim.options) {
  if (sim.options$doing.olave) {
    make.olave.alignments(sim.options$nofgenes, sim.options$taxa, sim.options$seqs.fpathbase)
  } else {
    make.alignments(sim.options$nofgenes, sim.options$seqs.fpathbase)
  }
}


# Read in the sequences files and join to make some XML
make.alignments <- function(nofgenes, seqs.fpathbase) {
  commentln("Taxa. Each taxon is a sequence (labelled A,B,... if more than one)")
  commentln("from an individual (01,02,03,...) in a species (a,b,c,...).")
  for (g in 1:nofgenes) {
    numastext <- as.character(g)
    textlines <- readLines(paste(seqs.fpathbase, numastext, ".txt", sep=""))
    n <- length(textlines)
    tokens <- strsplit(textlines, split=" +")    
    commentln(paste("Simulated alignment for gene", g, "of", nofgenes, "genes."))
    catln(1, "<data", id(alignmentID(g)), name("alignment"), ">")
    for (i in 2:n) {
      tx <- tokens[[i]][1]
      sq <- tokens[[i]][2]
      catln(2, "<sequence", id(paste0(tx, ".g", g)), 
            namevalue("taxon", tx), 
            namevalue("totalcount", "4"),
            namevalue("value", sq), "/>")
    }
    catln(1, "</data>")
  }
}




# Read in the sequences files and join to make some XML
make.olave.alignments <- function(nofgenes, taxa, seqs.fpathbase) {
  commentln("Taxa. Each taxon is a sequence (labelled A,B,...) from")
  commentln("an individual (01,02,03,...) in a species (a,b,c...).")
  for (g in 1:nofgenes) {
    numastext <- as.character(g)
    textlines <- readLines(paste(seqs.fpathbase, numastext, "-Seq.phy", sep=""))
    textlines <- gsub("^\\s+|\\s+$", "", textlines)
    n <- length(textlines)
    tokens <- strsplit(textlines, split="[[:space:]]+")
    commentln(paste("Olave et al's simulated alignment for gene", g, "of", nofgenes, "genes."))
    catln(1, "<data", id(alignmentID(g)), name("alignment"), ">")
    for (i in 2:n) {
      tx <- taxa[ as.integer(tokens[[i]][1]) ]
      sq <- tokens[[i]][2]
      catln(2, "<sequence", id(paste0(tx, ".g", g)),
            namevalue("taxon", tx), 
            namevalue("totalcount", "4"),
            namevalue("value", sq), "/>")
    }
    catln(1, "</data>")
  }
}

