#   Copyright Graham Jones 2014

#   This code is released under the Creative Commons 
#   Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)
#   licence, found here:
#   http://creativecommons.org/licenses/by-nc/4.0/


make.beast2xml.state <- function(sim.options, spp) {
  nofgenes <- sim.options$nofgenes
  catln(2, "<state", id("state"), namevalue("storeEvery", "500000"), ">")
  commentln("The species or minimal clusters tree (smcTree)")
  make.smctree(sim.options, spp)
  commentln("The birth-death-collapse hyper-parameters for smcTree")
  make.bdchyperparams()
  commentln("The hyper-parameter for the population scale for smcTree")
  make.poppriorscale()
  
  commentln("The gene trees")
  for (g in 1:nofgenes) {
    make.gtree(g)
  }
  commentln("The gene tree clock rates, omitting first")
  if (nofgenes > 1) {
    for (g in 2:nofgenes) {
      make.gtree.clockrate(g)
    }
  }

  commentln("The gene tree kappas for HKY")
  for (g in 1:nofgenes) {
    make.gtree.kappa(g)
  }
  commentln("The gene tree base frequencies")
  for (g in 1:nofgenes) {
    make.gtree.freqs(g)
  }
  catln(2, "</state>")
}


make.smctree <- function(sim.options, spp) {
  catln(3, "<tree", id(smcTreeID()), name("stateNode"), ">")
  make.taxon.setofsets(sim.options, spp)       
  catln(3, "</tree>")
}






make.taxon.setofsets <- function(sim.options, spp) {
  catln(4, "<taxonset", id(taxonSetOfSetsID()), spec("TaxonSet"), ">")
  msc.model <- sim.options$msc.model
  doing.olave <- sim.options$doing.olave
  taxa <- sim.options$taxa
  for (i in 1:length(spp)) {
    if (identical(msc.model , "STACEY")) {
      stopifnot(!doing.olave)
      sp <- spp[i]
    } else if (identical(msc.model , "STACEYD")) {
      if (doing.olave) {
        sp <- spp[i]
      } else {
        sp <- paste0(spp[i], "_mc")
      }     
    }   
    catln(5, "<taxon", id(sp), spec("TaxonSet"), ">")
    if (doing.olave) {
      catln(6, "<taxon", id(paste0(sp, "A")), spec("Taxon"), "/>")
      catln(6, "<taxon", id(paste0(sp, "B")), spec("Taxon"), "/>")
    } else {
      for (t in 1:length(taxa)) {
        if (taxon.belongs.to.species(msc.model, taxa[t], spp[i])) {
          catln(6, "<taxon", id(taxa[t]), spec("Taxon"), "/>")
        }
      }
    }
    catln(5, "</taxon>")
  } 
  catln(4, "</taxonset>")
}  




make.bdchyperparams <- function() {
  commentln("Initial value of growth rate may be used for initialising trees")
  commentln("and as a time scale in operators.")
  catln(3, "<parameter", 
        id(bdcGrowthID()), 
        namevalue("name", "stateNode"),
        namevalue("lower", "1.0E-99"),
        namevalue("upper", "1.0E99"),
        ">100.0</parameter>")
  catln(3, "<parameter", 
        id(bdcRelDeathID()), 
        name("stateNode"),
        namevalue("lower", "0.0"),
        namevalue("upper", "1.0"),
        ">0.5</parameter>")  
  catln(3, "<parameter", 
        id(bdcCollapseWtID()), 
        name("stateNode"),
        namevalue("lower", "0.0"),
        namevalue("upper", "1.0"),
        ">0.5</parameter>")
  commentln("Initial value of origin height is ignored")
  catln(3, "<parameter", 
        id(bdcOriginHtID()), 
        namevalue("name", "stateNode"),
        namevalue("lower", "1.0E-99"),
        namevalue("upper", "1.0E99"),
        ">100.0</parameter>")
}


make.poppriorscale <- function() {
  catln(3, "<parameter", 
      id(popSFID()), 
      name("stateNode"),
      namevalue("lower", "1.0E-9"),
      namevalue("upper", "1.0E9"),
      ">0.02</parameter>")
}


make.gtree <- function(g) {
  commentln(paste0("Gene tree ", g))
  catln(3, "<tree", id(geneTreeID(g)), name("stateNode"), ">") 
  catln(4, "<taxonset",
        id(geneTaxonSetID(g)), 
        namevalue("spec", "TaxonSet"), ">")
  catln(5, idref("data", alignmentID(g), "alignment"))
  catln(4, "</taxonset>")
  catln(3, "</tree>")
}



make.gtree.clockrate <- function(g) {
  catln(3, "<parameter", id(clockRateID(g)), name("stateNode"), ">1.0</parameter>")  
}

make.gtree.kappa <- function(g) {
  catln(3, "<parameter", 
        id(kappaID(g)), 
        name("stateNode"), 
        ">2.0</parameter>")  
}

  
make.gtree.freqs <- function(g) {
  catln(3, "<parameter", 
        id(frequenciesID(g)),
        namevalue("dimension", "4"),
        name("stateNode"), 
        namevalue("lower", "0.0"),
        namevalue("upper", "1.0"),
        ">0.25</parameter>")    
}




taxon.belongs.to.species <- function(msc.model, taxon, sp) {
  if (identical(msc.model , "STACEY")) {
    identical(substr(taxon, 1, 1), sp)
  } else if (identical(msc.model , "STACEYD")) {
    identical(taxon, sp)
  }
}









